@@ -2,12 +2,12 @@ from __future__ import annotations
 
 import contextlib
 import itertools
+import pickle
 import sys
 import warnings
 from numbers import Number
 
 import pytest
-from numpy import AxisError
 
 import dask
 from dask.delayed import delayed
@@ -15,7 +15,7 @@ from dask.delayed import delayed
 np = pytest.importorskip("numpy")
 
 import dask.array as da
-from dask.array.numpy_compat import _numpy_123
+from dask.array.numpy_compat import NUMPY_GE_200, AxisError
 from dask.array.utils import assert_eq, same_keys
 
 
@@ -334,7 +334,7 @@ def test_tensordot():
     y = np.arange(200).reshape((20, 10))
     b = da.from_array(y, chunks=(4, 5))
 
-    for axes in [1, (1, 0)]:
+    for axes in [1, (1, 0), (-1, 0)]:
         assert_eq(da.tensordot(a, b, axes=axes), np.tensordot(x, y, axes=axes))
         assert_eq(da.tensordot(x, b, axes=axes), np.tensordot(x, y, axes=axes))
         assert_eq(da.tensordot(a, y, axes=axes), np.tensordot(x, y, axes=axes))
@@ -1277,28 +1277,23 @@ def test_unique_kwargs(return_index, return_inverse, return_counts):
 
 
 @pytest.mark.parametrize("seed", [23, 796])
-@pytest.mark.parametrize("low, high", [[0, 10]])
 @pytest.mark.parametrize(
     "shape, chunks",
     [[(10,), (5,)], [(10,), (3,)], [(4, 5), (3, 2)], [(20, 20), (4, 5)]],
 )
-def test_unique_rand(seed, low, high, shape, chunks):
+def test_unique_rand(seed, shape, chunks):
     rng = np.random.default_rng(seed)
 
-    a = rng.integers(low, high, size=shape)
+    a = rng.integers(0, 10, size=shape)
     d = da.from_array(a, chunks=chunks)
 
-    kwargs = dict(return_index=True, return_inverse=True, return_counts=True)
+    r_a = np.unique(a, return_index=True, return_inverse=True, return_counts=True)
+    r_d = da.unique(d, return_index=True, return_inverse=True, return_counts=True)
 
-    r_a = np.unique(a, **kwargs)
-    r_d = da.unique(d, **kwargs)
-
-    assert len(r_a) == len(r_d)
-
-    assert (d.size,) == r_d[2].shape
-
-    for e_r_a, e_r_d in zip(r_a, r_d):
-        assert_eq(e_r_d, e_r_a)
+    assert_eq(r_d[0], r_a[0])
+    assert_eq(r_d[1], r_a[1])
+    assert_eq(r_d[2], r_a[2])
+    assert_eq(r_d[3], r_a[3])
 
 
 @pytest.mark.parametrize("seed", [23, 796])
@@ -1962,7 +1957,7 @@ def test_count_nonzero_str():
     # We may have behavior differences with NumPy for strings
     # with just spaces, depending on the version of NumPy.
     # https://github.com/numpy/numpy/issues/9875
-    x = np.array(list("Hellow orld"))
+    x = np.array(list("Hello world"))
     d = da.from_array(x, chunks=(4,))
 
     x_c = np.count_nonzero(x)
@@ -2252,10 +2247,10 @@ def test_insert():
     with pytest.raises(NotImplementedError):
         da.insert(a, [4, 2], -1, axis=0)
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.insert(a, [3], -1, axis=2)
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.insert(a, [3], -1, axis=-3)
 
 
@@ -2298,9 +2293,9 @@ def test_append():
     )
 
     # check AxisError
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.append(a, ((0,) * 10,) * 10, axis=2)
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.append(a, ((0,) * 10,) * 10, axis=-3)
 
     # check ValueError if dimensions don't align
@@ -2337,10 +2332,10 @@ def test_delete():
 
     assert_eq(np.delete(a, [4, 2], axis=0), da.delete(a, [4, 2], axis=0))
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.delete(a, [3], axis=2)
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         da.delete(a, [3], axis=-3)
 
 
@@ -2357,9 +2352,15 @@ def test_result_type():
     # Effect of scalars depends on their value
     assert da.result_type(1, b) == np.int16
     assert da.result_type(1.0, a) == np.float32
-    assert da.result_type(np.int64(1), b) == np.int16
-    assert da.result_type(np.ones((), np.int64), b) == np.int16  # 0d array
-    assert da.result_type(1e200, a) == np.float64  # 1e200 is too big for float32
+    if NUMPY_GE_200:
+        assert da.result_type(np.int64(1), b) == np.int64
+        assert da.result_type(np.ones((), np.int64), b) == np.int64
+        assert da.result_type(1e200, a) == np.float32
+    else:
+        assert da.result_type(np.int64(1), b) == np.int16
+        assert da.result_type(np.ones((), np.int64), b) == np.int16  # 0d array
+        assert da.result_type(1e200, a) == np.float64  # 1e200 is too big for float32
+
     # dask 0d-arrays are NOT treated like scalars
     c = da.from_array(np.ones((), np.float64), chunks=())
     assert da.result_type(a, c) == np.float64
@@ -2594,9 +2595,8 @@ def test_average_keepdims(a):
 
     da_avg = da.average(d_a, keepdims=True)
 
-    if _numpy_123:
-        np_avg = np.average(a, keepdims=True)
-        assert_eq(np_avg, da_avg)
+    np_avg = np.average(a, keepdims=True)
+    assert_eq(np_avg, da_avg)
 
 
 @pytest.mark.parametrize("keepdims", [False, True])
@@ -2609,10 +2609,7 @@ def test_average_weights(keepdims):
 
     da_avg = da.average(d_a, weights=d_weights, axis=1, keepdims=keepdims)
 
-    if _numpy_123:
-        assert_eq(da_avg, np.average(a, weights=weights, axis=1, keepdims=keepdims))
-    elif not keepdims:
-        assert_eq(da_avg, np.average(a, weights=weights, axis=1))
+    assert_eq(da_avg, np.average(a, weights=weights, axis=1, keepdims=keepdims))
 
 
 def test_average_raises():
@@ -2708,3 +2705,18 @@ def test_tril_triu_indices(n, k, m, chunks):
         )
     else:
         assert_eq(actual, expected)
+
+
+def test_pickle_vectorized_routines():
+    """Test that graphs that internally use np.vectorize can be pickled"""
+    a = da.from_array(["foo", "bar", ""])
+
+    b = da.count_nonzero(a)
+    assert_eq(b, 2, check_dtype=False)
+    b2 = pickle.loads(pickle.dumps(b))
+    assert_eq(b2, 2, check_dtype=False)
+
+    c = da.argwhere(a)
+    assert_eq(c, [[0], [1]], check_dtype=False)
+    c2 = pickle.loads(pickle.dumps(c))
+    assert_eq(c2, [[0], [1]], check_dtype=False)
