@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 import itertools
 import warnings
 
 import pytest
-from tlz import merge
+
+from dask.array._shuffle import concatenate_arrays
 
 np = pytest.importorskip("numpy")
 
@@ -19,7 +22,6 @@ from dask.array.slicing import (
     sanitize_index,
     shuffle_slice,
     slice_array,
-    slicing_plan,
     take,
 )
 from dask.array.utils import assert_eq, same_keys
@@ -313,62 +315,111 @@ def test_slicing_with_newaxis():
 
 
 def test_take():
-    chunks, dsk = take("y", "x", [(20, 20, 20, 20)], [5, 1, 47, 3], itemsize=8, axis=0)
+    chunks, dsk = take("y-y", "x", [(20, 20, 20, 20)], [5, 1, 47, 3], axis=0)
     expected = {
-        ("y", 0): (getitem, ("x", 0), (np.array([5, 1]),)),
-        ("y", 1): (getitem, ("x", 2), (np.array([7]),)),
-        ("y", 2): (getitem, ("x", 0), (np.array([3]),)),
+        ("y-y", 0): (
+            concatenate_arrays,
+            [
+                ("shuffle-split-y", 0),
+                ("shuffle-split-y", 1),
+            ],
+            np.array([1, 3, 0, 2]),
+            0,
+        ),
+        ("shuffle-split-y", 0): (
+            getitem,
+            ("x", 0),
+            (np.array([1, 3, 5]),),
+        ),
+        ("shuffle-split-y", 1): (
+            getitem,
+            ("x", 2),
+            (np.array([7]),),
+        ),
     }
     np.testing.assert_equal(sorted(dsk.items()), sorted(expected.items()))
-    assert chunks == ((2, 1, 1),)
+    assert chunks == ((4,),)
 
-    chunks, dsk = take(
-        "y", "x", [(20, 20, 20, 20), (20, 20)], [5, 1, 47, 3], itemsize=8, axis=0
-    )
+    chunks, dsk = take("y-y", "x", [(20, 20, 20, 20), (20, 20)], [5, 1, 47, 3], axis=0)
     expected = {
-        ("y", 0, 0): (
+        ("y-y", 0, 0): (
+            concatenate_arrays,
+            [
+                ("shuffle-split-y", 0),
+                ("shuffle-split-y", 1),
+            ],
+            np.array([1, 3, 0, 2]),
+            0,
+        ),
+        ("y-y", 0, 1): (
+            concatenate_arrays,
+            [
+                ("shuffle-split-y", 2),
+                ("shuffle-split-y", 3),
+            ],
+            np.array([1, 3, 0, 2]),
+            0,
+        ),
+        ("shuffle-split-y", 0): (
             getitem,
             ("x", 0, 0),
-            (np.array([5, 1]), slice(None, None, None)),
+            (np.array([1, 3, 5]), slice(None)),
         ),
-        ("y", 0, 1): (
+        ("shuffle-split-y", 1): (
+            getitem,
+            ("x", 2, 0),
+            (np.array([7]), slice(None)),
+        ),
+        ("shuffle-split-y", 2): (
             getitem,
             ("x", 0, 1),
-            (np.array([5, 1]), slice(None, None, None)),
+            (np.array([1, 3, 5]), slice(None)),
+        ),
+        ("shuffle-split-y", 3): (
+            getitem,
+            ("x", 2, 1),
+            (np.array([7]), slice(None)),
         ),
-        ("y", 1, 0): (getitem, ("x", 2, 0), (np.array([7]), slice(None, None, None))),
-        ("y", 1, 1): (getitem, ("x", 2, 1), (np.array([7]), slice(None, None, None))),
-        ("y", 2, 0): (getitem, ("x", 0, 0), (np.array([3]), slice(None, None, None))),
-        ("y", 2, 1): (getitem, ("x", 0, 1), (np.array([3]), slice(None, None, None))),
     }
     np.testing.assert_equal(sorted(dsk.items()), sorted(expected.items()))
-    assert chunks == ((2, 1, 1), (20, 20))
+    assert chunks == ((4,), (20, 20))
 
 
 def test_take_sorted():
-    chunks, dsk = take("y", "x", [(20, 20, 20, 20)], [1, 3, 5, 47], itemsize=8, axis=0)
+    chunks, dsk = take("y-y", "x", [(20, 20, 20, 20)], [1, 3, 5, 47], axis=0)
     expected = {
-        ("y", 0): (getitem, ("x", 0), ([1, 3, 5],)),
-        ("y", 1): (getitem, ("x", 2), ([7],)),
+        ("y-y", 0): (
+            concatenate_arrays,
+            [
+                ("shuffle-split-y", 0),
+                ("shuffle-split-y", 1),
+            ],
+            np.array([0, 1, 2, 3]),
+            0,
+        ),
+        ("shuffle-split-y", 0): (
+            getitem,
+            ("x", 0),
+            (np.array([1, 3, 5]),),
+        ),
+        ("shuffle-split-y", 1): (
+            getitem,
+            ("x", 2),
+            (np.array([7]),),
+        ),
     }
     np.testing.assert_equal(dsk, expected)
-    assert chunks == ((3, 1),)
+    assert chunks == ((4,),)
 
-    chunks, dsk = take(
-        "y", "x", [(20, 20, 20, 20), (20, 20)], [1, 3, 5, 37], itemsize=8, axis=1
-    )
-    expected = merge(
-        {
-            ("y", i, 0): (getitem, ("x", i, 0), (slice(None, None, None), [1, 3, 5]))
-            for i in range(4)
-        },
-        {
-            ("y", i, 1): (getitem, ("x", i, 1), (slice(None, None, None), [17]))
-            for i in range(4)
-        },
-    )
+    chunks, dsk = take("y", "x", [(20, 20, 20, 20)], np.arange(0, 80), axis=0)
+    expected = {
+        ("y", 0): ("x", 0),
+        ("y", 1): ("x", 1),
+        ("y", 2): ("x", 2),
+        ("y", 3): ("x", 3),
+    }
     np.testing.assert_equal(dsk, expected)
-    assert chunks == ((20, 20, 20, 20), (3, 1))
+    assert chunks == ((20, 20, 20, 20),)
 
 
 def test_slicing_chunks():
@@ -390,14 +441,14 @@ def test_slicing_chunks():
 
 def test_slicing_with_numpy_arrays():
     a, bd1 = slice_array(
-        "y",
+        "y-y",
         "x",
         ((3, 3, 3, 1), (3, 3, 3, 1)),
         (np.array([1, 2, 9]), slice(None, None, None)),
         itemsize=8,
     )
     b, bd2 = slice_array(
-        "y",
+        "y-y",
         "x",
         ((3, 3, 3, 1), (3, 3, 3, 1)),
         (np.array([1, 2, 9]), slice(None, None, None)),
@@ -410,7 +461,7 @@ def test_slicing_with_numpy_arrays():
     i = [False, True, True, False, False, False, False, False, False, True]
     index = (i, slice(None, None, None))
     index = normalize_index(index, (10, 10))
-    c, bd3 = slice_array("y", "x", ((3, 3, 3, 1), (3, 3, 3, 1)), index, itemsize=8)
+    c, bd3 = slice_array("y-y", "x", ((3, 3, 3, 1), (3, 3, 3, 1)), index, itemsize=8)
     assert bd1 == bd3
     np.testing.assert_equal(a, c)
 
@@ -464,26 +515,26 @@ class ReturnItem:
 
 @pytest.mark.skip(reason="really long test")
 def test_slicing_exhaustively():
-    x = np.random.rand(6, 7, 8)
+    x = np.random.default_rng().random(6, 7, 8)
     a = da.from_array(x, chunks=(3, 3, 3))
     I = ReturnItem()
 
     # independent indexing along different axes
     indexers = [0, -2, I[:], I[:5], [0, 1], [0, 1, 2], [4, 2], I[::-1], None, I[:0], []]
     for i in indexers:
-        assert_eq(x[i], a[i]), i
+        assert_eq(x[i], a[i])
         for j in indexers:
-            assert_eq(x[i][:, j], a[i][:, j]), (i, j)
-            assert_eq(x[:, i][j], a[:, i][j]), (i, j)
+            assert_eq(x[i][:, j], a[i][:, j])
+            assert_eq(x[:, i][j], a[:, i][j])
             for k in indexers:
-                assert_eq(x[..., i][:, j][k], a[..., i][:, j][k]), (i, j, k)
+                assert_eq(x[..., i][:, j][k], a[..., i][:, j][k])
 
     # repeated indexing along the first axis
     first_indexers = [I[:], I[:5], np.arange(5), [3, 1, 4, 5, 0], np.arange(6) < 6]
     second_indexers = [0, -1, 3, I[:], I[:3], I[2:-1], [2, 4], [], I[:0]]
     for i in first_indexers:
         for j in second_indexers:
-            assert_eq(x[i][j], a[i][j]), (i, j)
+            assert_eq(x[i][j], a[i][j])
 
 
 def test_slicing_with_negative_step_flops_keys():
@@ -508,7 +559,7 @@ def test_empty_slice():
 
 
 def test_multiple_list_slicing():
-    x = np.random.rand(6, 7, 8)
+    x = np.random.default_rng().random((6, 7, 8))
     a = da.from_array(x, chunks=(3, 3, 3))
     assert_eq(x[:, [0, 1, 2]][[0, 1]], a[:, [0, 1, 2]][[0, 1]])
 
@@ -727,8 +778,9 @@ def test_index_with_bool_dask_array():
 
 
 def test_index_with_bool_dask_array_2():
-    x = np.random.random((10, 10, 10))
-    ind = np.random.random(10) > 0.5
+    rng = np.random.default_rng()
+    x = rng.random((10, 10, 10))
+    ind = rng.random(10) > 0.5
 
     d = da.from_array(x, chunks=(3, 4, 5))
     dind = da.from_array(ind, chunks=4)
@@ -759,7 +811,7 @@ def test_cull():
     "index", [(Ellipsis,), (None, Ellipsis), (Ellipsis, None), (None, Ellipsis, None)]
 )
 def test_slicing_with_Nones(shape, index):
-    x = np.random.random(shape)
+    x = np.random.default_rng().random(shape)
     d = da.from_array(x, chunks=shape)
 
     assert_eq(x[index], d[index])
@@ -789,7 +841,7 @@ def test_slicing_none_int_ellipses(a, b, c, d):
 
 def test_slicing_integer_no_warnings():
     # https://github.com/dask/dask/pull/2457/
-    X = da.random.random((100, 2), (2, 2))
+    X = da.random.default_rng().random(size=(100, 2), chunks=(2, 2))
     idx = np.array([0, 0, 1, 1])
     with warnings.catch_warnings(record=True) as record:
         X[idx].compute()
@@ -857,27 +909,6 @@ def test_take_semi_sorted():
     assert y.chunks == ((5, 5, 5),)
 
 
-@pytest.mark.parametrize(
-    "chunks,index,expected",
-    [
-        ((5, 5, 5), np.arange(5, 15) % 10, [(1, np.arange(5)), (0, np.arange(5))]),
-        (
-            (5, 5, 5, 5),
-            np.arange(20) // 2,
-            [(0, np.arange(10) // 2), (1, np.arange(10) // 2)],
-        ),
-        ((10, 10), [15, 2, 3, 15], [(1, [5]), (0, [2, 3]), (1, [5])]),
-    ],
-)
-def test_slicing_plan(chunks, index, expected):
-    plan = slicing_plan(chunks, index=index)
-    assert len(plan) == len(expected)
-    for (i, x), (j, y) in zip(plan, expected):
-        assert i == j
-        assert len(x) == len(y)
-        assert (x == y).all()
-
-
 def test_getitem_avoids_large_chunks():
     with dask.config.set({"array.chunk-size": "0.1Mb"}):
         a = np.arange(2 * 128 * 128, dtype="int64").reshape(2, 128, 128)
@@ -892,12 +923,9 @@ def test_getitem_avoids_large_chunks():
         arr = da.from_array(a, chunks=(1, 128, 128))  # large chunks
         expected = a[indexer]
 
-        # By default, we warn
-        with pytest.warns(da.PerformanceWarning):
-            result = arr[indexer]
-
+        result = arr[indexer]
         assert_eq(result, expected)
-        assert result.chunks == ((1, 11), (128,), (128,))
+        assert result.chunks == ((1,) * 12, (128,), (128,))
 
         # Users can silence the warning
         with dask.config.set({"array.slicing.split-large-chunks": False}):
@@ -916,23 +944,13 @@ def test_getitem_avoids_large_chunks():
             assert result.chunks == ((1,) * 12, (128,), (128,))
 
 
-@pytest.mark.parametrize(
-    "chunks",
-    [
-        ((1, 1, 1, 1), (np.nan,), (np.nan,)),
-        pytest.param(
-            ((np.nan, np.nan, np.nan, np.nan), (500,), (500,)),
-            marks=pytest.mark.xfail(reason="https://github.com/dask/dask/issues/6586"),
-        ),
-    ],
-)
-def test_getitem_avoids_large_chunks_missing(chunks):
+def test_getitem_avoids_large_chunks_missing():
     # We cannot apply the "avoid large chunks" optimization when
     # the chunks have unknown sizes.
-    with dask.config.set({"array.slicing.split-large-chunks": True}):
+    with dask.config.set({"array.chunk-size": "0.1Mb"}):
         a = np.arange(4 * 500 * 500).reshape(4, 500, 500)
         arr = da.from_array(a, chunks=(1, 500, 500))
-        arr._chunks = chunks
+        arr._chunks = ((1, 1, 1, 1), (np.nan,), (np.nan,))
         indexer = [0, 1] + [2] * 100 + [3]
         expected = a[indexer]
         result = arr[indexer]
@@ -943,38 +961,36 @@ def test_take_avoids_large_chunks():
     # unit test for https://github.com/dask/dask/issues/6270
     with dask.config.set({"array.slicing.split-large-chunks": True}):
         chunks = ((1, 1, 1, 1), (500,), (500,))
-        itemsize = 8
         index = np.array([0, 1] + [2] * 101 + [3])
-        chunks2, dsk = take("a", "b", chunks, index, itemsize)
-        assert chunks2 == ((1, 1, 51, 50, 1), (500,), (500,))
-        assert len(dsk) == 5
+        chunks2, dsk = take("a", "b", chunks, index)
+        assert chunks2 == ((1,) * 104, (500,), (500,))
+        assert len(dsk) == 104
 
         index = np.array([0] * 101 + [1, 2, 3])
-        chunks2, dsk = take("a", "b", chunks, index, itemsize)
-        assert chunks2 == ((51, 50, 1, 1, 1), (500,), (500,))
-        assert len(dsk) == 5
+        chunks2, dsk = take("a", "b", chunks, index)
+        assert chunks2 == ((1,) * 104, (500,), (500,))
+        assert len(dsk) == 104
 
         index = np.array([0, 1, 2] + [3] * 101)
-        chunks2, dsk = take("a", "b", chunks, index, itemsize)
-        assert chunks2 == ((1, 1, 1, 51, 50), (500,), (500,))
-        assert len(dsk) == 5
+        chunks2, dsk = take("a", "b", chunks, index)
+        assert chunks2 == ((1,) * 104, (500,), (500,))
+        assert len(dsk) == 104
 
         chunks = ((500,), (1, 1, 1, 1), (500,))
         index = np.array([0, 1, 2] + [3] * 101)
-        chunks2, dsk = take("a", "b", chunks, index, itemsize, axis=1)
-        assert chunks2 == ((500,), (1, 1, 1, 51, 50), (500,))
-        assert len(dsk) == 5
+        chunks2, dsk = take("a", "b", chunks, index, axis=1)
+        assert chunks2 == ((500,), (1,) * 104, (500,))
+        assert len(dsk) == 104
 
 
 def test_take_uses_config():
     with dask.config.set({"array.slicing.split-large-chunks": True}):
         chunks = ((1, 1, 1, 1), (500,), (500,))
         index = np.array([0, 1] + [2] * 101 + [3])
-        itemsize = 8
         with config.set({"array.chunk-size": "10GB"}):
-            chunks2, dsk = take("a", "b", chunks, index, itemsize)
-        assert chunks2 == ((1, 1, 101, 1), (500,), (500,))
-        assert len(dsk) == 4
+            chunks2, dsk = take("a", "b", chunks, index)
+        assert chunks2 == ((1,) * 104, (500,), (500,))
+        assert len(dsk) == 104
 
 
 def test_pathological_unsorted_slicing():
@@ -983,11 +999,7 @@ def test_pathological_unsorted_slicing():
     # [0, 10, 20, ... 90, 1, 11, 21, ... 91, ...]
     index = np.arange(100).reshape(10, 10).ravel(order="F")
 
-    with pytest.warns(da.PerformanceWarning) as info:
-        x[index]
-
-    assert "10" in str(info.list[0])
-    assert "out-of-order" in str(info.list[0])
+    assert_eq(x[index], x.compute()[index])
 
 
 @pytest.mark.parametrize("params", [(2, 2, 1), (5, 3, 2)])
@@ -1028,15 +1040,38 @@ def test_make_blockwise_sorted_slice():
     "size, chunks", [((100, 2), (50, 2)), ((100, 2), (37, 1)), ((100,), (55,))]
 )
 def test_shuffle_slice(size, chunks):
-    x = da.random.randint(0, 1000, size=size, chunks=chunks)
+    x = da.random.default_rng().integers(0, 1000, size=size, chunks=chunks)
     index = np.arange(len(x))
-    np.random.shuffle(index)
+    np.random.default_rng().shuffle(index)
 
     a = x[index]
     b = shuffle_slice(x, index)
     assert_eq(a, b)
 
 
+def test_unknown_chunks_length_one():
+    a = np.arange(256, dtype=int)
+    arr = da.from_array(a, chunks=(256,))
+    # np.flatnonzero dispatches
+    result = np.flatnonzero(arr)
+    assert_eq(result[[0, -1]], np.flatnonzero(a)[[0, -1]])
+
+    result = da.flatnonzero(arr)
+    assert_eq(result[[0, -1]], np.flatnonzero(a)[[0, -1]])
+
+    a = a.reshape(16, 16)
+    arr = da.from_array(a, chunks=(8, 16))
+    arr._chunks = ((8, 8), (np.nan,))
+    result = arr[:, [0, -1]]
+    expected = a[:, [0, -1]]
+    assert_eq(result, expected)
+
+    arr = da.from_array(a, chunks=(8, 8))
+    arr._chunks = ((8, 8), (np.nan, np.nan))
+    with pytest.raises(ValueError, match="Array chunk size or shape"):
+        arr[:, [0, -1]]
+
+
 @pytest.mark.parametrize("lock", [True, False])
 @pytest.mark.parametrize("asarray", [True, False])
 @pytest.mark.parametrize("fancy", [True, False])
@@ -1057,7 +1092,30 @@ def test_slice_array_3d_with_bool_numpy_array():
     assert_eq(actual, expected)
 
 
+def test_slice_masked_arrays():
+    arr = np.ma.array(range(8), mask=[0, 0, 1, 0, 0, 1, 0, 1])
+    darr = da.from_array(arr, chunks=(4, 4))
+    assert_eq(darr[[2, 6]], arr[[2, 6]])
+
+
 def test_slice_array_null_dimension():
     array = da.from_array(np.zeros((3, 0)))
     expected = np.zeros((3, 0))[[0]]
     assert_eq(array[[0]], expected)
+
+
+def test_take_sorted_indexer():
+    arr = da.ones((250, 100), chunks=((50, 100, 33, 67), 100))
+    indexer = list(range(0, 250))
+    result = arr[indexer, :]
+    assert_eq(arr, result)
+    assert {
+        **dict(arr.dask),
+        **{
+            k: k2
+            for k, k2 in zip(
+                [k for k in dict(result.dask) if "getitem" in k[0]],
+                dict(arr.dask).keys(),
+            )
+        },
+    } == dict(result.dask)
